Index: channels/chan_agent.c
===================================================================
--- channels/chan_agent.c	(revision 7982)
+++ channels/chan_agent.c	(working copy)
@@ -157,6 +157,7 @@
 static int autologoff;
 static int wrapuptime;
 static int ackcall;
+static int multipelogin;
 
 static int maxlogintries = 3;
 static char agentgoodbye[AST_MAX_FILENAME_LEN] = "vm-goodbye";
@@ -188,6 +189,7 @@
 	int abouttograb;               /**< About to grab */
 	int autologoff;                /**< Auto timeout time */
 	int ackcall;                   /**< ackcall */
+	int multipelogin;              /**< multipelogin */
 	time_t loginstart;             /**< When agent first logged in (0 when logged off) */
 	time_t start;                  /**< When call started */
 	struct timeval lastdisc;       /**< When last disconnected */
@@ -371,6 +373,7 @@
 	ast_copy_string(p->moh, moh, sizeof(p->moh));
 	p->ackcall = ackcall;
 	p->autologoff = autologoff;
+	p->multipelogin = multipelogin;
 
 	/* If someone reduces the wrapuptime and reloads, we want it
 	 * to change the wrapuptime immediately on all calls */
@@ -1023,6 +1026,7 @@
 	autologoff = 0;
 	wrapuptime = 0;
 	ackcall = 0;
+	multipelogin = 0;
 	cfg = ast_config_load(config);
 	if (!cfg) {
 		ast_log(LOG_NOTICE, "No agent configuration found -- agent support disabled\n");
@@ -1106,6 +1110,8 @@
 				strncat(savecallsin, "/", sizeof(savecallsin) - strlen(savecallsin) - 1);
 		} else if (!strcasecmp(v->name, "custom_beep")) {
 			ast_copy_string(beep, v->value, sizeof(beep));
+		} else if (!strcasecmp(v->name, "multipelogin")) {
+			multipelogin = ast_true(v->value);
 		}
 		v = v->next;
 	}
@@ -1258,6 +1264,26 @@
 	return res;
 }
 
+/* return 0 if there are no same loginchan name */
+static int check_extension(char *chan, char *context)
+{
+	struct agent_pvt *p;
+	char loginchan[80];
+
+	if(!chan || !context)
+		return 0;
+
+	snprintf(loginchan, sizeof(loginchan), "%s@%s", chan, !ast_strlen_zero(context) ? context : "default");
+
+	p = agents;
+	while(p) {
+		if(!p->multipelogin && !strcasecmp(loginchan, p->loginchan))
+			return -1;
+		p = p->next;
+	}
+	return 0;
+}
+
 /*--- agent_request: Part of the Asterisk PBX interface ---*/
 static struct ast_channel *agent_request(const char *type, int format, void *data, int *cause)
 {
@@ -1826,9 +1852,11 @@
 								res = 0;
 							} else
 								res = ast_app_getdata(chan, "agent-newlocation", tmpchan+pos, sizeof(tmpchan) - 2, 0);
-							if (ast_strlen_zero(tmpchan) || ast_exists_extension(chan, !ast_strlen_zero(context) ? context : "default", tmpchan,
-													     1, NULL))
+							if (ast_strlen_zero(tmpchan))
 								break;
+							
+							if (!check_extension(chan, context) && ast_exists_extension(chan, !ast_strlen_zero(context) ? context : "default", tmpchan, 1, NULL))
+								break;
 							if (args.extension) {
 								ast_log(LOG_WARNING, "Extension '%s' is not valid for automatic login of agent '%s'\n", args.extension, p->agent);
 								args.extension = NULL;
